
"use client";

import { useRef, useState, useEffect, useActionState } from 'react'; // Updated import
import { useFormStatus } from 'react-dom';
import { uploadFileAction } from '@/lib/actions/file.actions';
import { Button } from '@/components/ui/button';
import { Input } from '@/components/ui/input';
import { Label } from '@/components/ui/label';
import { Progress } from '@/components/ui/progress';
import { useToast } from '@/hooks/use-toast';
import { Alert, AlertDescription, AlertTitle } from "@/components/ui/alert";
import { UploadCloud, CheckCircle, XCircle, Loader2, File as FileIconLucide } from "lucide-react";

function SubmitButton() {
  const { pending } = useFormStatus();
  return (
    <Button type="submit" className="w-full mt-4" disabled={pending}>
      {pending ? <Loader2 className="mr-2 h-4 w-4 animate-spin" /> : <UploadCloud className="mr-2 h-4 w-4" />}
      Upload File
    </Button>
  );
}

export default function FileUploadForm() {
  const initialState = { message: null, error: false, file: null };
  // Updated to useActionState
  const [state, dispatch, isPending] = useActionState(uploadFileAction, initialState); 
  const { toast } = useToast();
  const [selectedFile, setSelectedFile] = useState<File | null>(null);
  const [uploadProgress, setUploadProgress] = useState(0); // Mock progress
  const formRef = useRef<HTMLFormElement>(null);

  const handleFileChange = (event: React.ChangeEvent<HTMLInputElement>) => {
    const file = event.target.files?.[0];
    if (file) {
      setSelectedFile(file);
      setUploadProgress(0); // Reset progress
    } else {
      setSelectedFile(null);
    }
  };
  
  // Simulate upload progress for UX
  useEffect(() => {
    let interval: NodeJS.Timeout;
    // Check if isPending (from useActionState) is true and selectedFile exists
    if (isPending && selectedFile && !state?.message) { 
      let progress = 0;
      // Simulate progress quickly at first, then slower
      interval = setInterval(() => {
        progress += Math.random() * 15 + 5; // Random progress increment
        if (progress >= 95 && !state?.message) { // Stall at 95% if no response yet
           setUploadProgress(95);
        } else if (progress <= 100) {
          setUploadProgress(Math.min(progress, 100));
        }
        if (progress >= 100 || state?.message) {
           clearInterval(interval);
        }
      }, 150);
    } else if (state?.message && !state.error) {
      // If successful, jump to 100%
      setUploadProgress(100);
    } else if (state?.message && state.error) {
      // If error, reset progress or show error state (e.g., 0 or specific error indicator)
      setUploadProgress(0);
    }
    return () => clearInterval(interval);
  }, [isPending, selectedFile, state]);


  useEffect(() => {
    if (state?.message) {
      if (state.error) {
        toast({
          variant: "destructive",
          title: "Upload Failed",
          description: state.message,
          icon: <XCircle className="h-5 w-5" />,
        });
      } else {
        toast({
          title: "Upload Successful!",
          description: state.message,
          icon: <CheckCircle className="h-5 w-5 text-green-500" />,
        });
        formRef.current?.reset(); 
        setSelectedFile(null); 
        setUploadProgress(100); 
        // Optionally reset progress to 0 after a short delay if user stays on page
        setTimeout(() => setUploadProgress(0), 2000);
      }
    }
  }, [state, toast]);

  return (
    <form action={dispatch} ref={formRef} className="space-y-6">
      <div>
        <Label htmlFor="file" className="sr-only">Choose file</Label>
        <div className="mt-2 flex justify-center rounded-lg border border-dashed border-border px-6 py-10 hover:border-primary transition-colors duration-200">
          <div className="text-center">
            {selectedFile ? (
              <FileIconLucide className="mx-auto h-12 w-12 text-muted-foreground" aria-hidden="true" />
            ) : (
              <UploadCloud className="mx-auto h-12 w-12 text-muted-foreground" aria-hidden="true" />
            )}
            <div className="mt-4 flex text-sm leading-6 text-muted-foreground">
              <label
                htmlFor="file-upload"
                className="relative cursor-pointer rounded-md font-semibold text-primary focus-within:outline-none focus-within:ring-2 focus-within:ring-primary focus-within:ring-offset-2 hover:text-accent"
              >
                <span>{selectedFile ? "Change file" : "Upload a file"}</span>
                <Input id="file-upload" name="file" type="file" className="sr-only" onChange={handleFileChange} required />
              </label>
              <p className="pl-1">{selectedFile ? "" : "or drag and drop"}</p>
            </div>
            {selectedFile ? (
              <p className="text-xs leading-5 text-muted-foreground mt-1">{selectedFile.name} ({(selectedFile.size / 1024 / 1024).toFixed(2)} MB)</p>
            ) : (
              <p className="text-xs leading-5 text-muted-foreground">PNG, JPG, PDF, MP4 up to 50MB</p>
            )}
          </div>
        </div>
      </div>

      {selectedFile && (isPending || uploadProgress > 0) && (
        <div className="space-y-2">
          <Progress value={uploadProgress} className="w-full h-3" />
          <p className="text-sm text-muted-foreground text-center">
            {isPending && uploadProgress < 100 ? `${uploadProgress}% uploading...` : state?.error ? 'Error' : uploadProgress === 100 && !isPending ? 'Complete' : `${uploadProgress}%`}
          </p>
        </div>
      )}
      
      {state?.error && state.message && !isPending && (
          <Alert variant="destructive">
            <XCircle className="h-4 w-4" />
            <AlertTitle>Error</AlertTitle>
            <AlertDescription>{state.message}</AlertDescription>
          </Alert>
        )}

      <SubmitButton />
    </form>
  );
}
