
"use server";

import { addFile } from '@/lib/file-service';
import { revalidatePath } from 'next/cache';
import { z } from 'zod';
import fs from 'node:fs';
import path from 'node:path';

const MAX_FILE_SIZE = 50 * 1024 * 1024; // 50MB
// It's generally better to allow a wider range of types and let the application handle them,
// or use more robust server-side type checking if strict limitations are needed.
// For this prototype, we'll be more permissive on the client-side Zod schema for MIME types.
// const ACCEPTED_FILE_TYPES_REGEX = /^(image\/[a-zA-Z0-9+.-]+|application\/pdf|video\/[a-zA-Z0-9+.-]+|audio\/[a-zA-Z0-9+.-]+|text\/[a-zA-Z0-9+.-]+|application\/(json|zip|x-rar-compressed|octet-stream))$/;


const fileSchema = z.object({
  file: z
    .custom<File>((val) => val instanceof File, "Input is not a file")
    .refine((file) => file.size > 0, "File cannot be empty.")
    .refine((file) => file.size <= MAX_FILE_SIZE, `File size should be less than 50MB.`)
    // .refine((file) => ACCEPTED_FILE_TYPES_REGEX.test(file.type), "Unsupported file type.") // Relaxing this for broader prototype compatibility
});


export async function uploadFileAction(prevState: any, formData: FormData) {
  const uploadedFile = formData.get('file') as File;

  if (!uploadedFile || uploadedFile.name === 'undefined' || uploadedFile.size === 0) { // Handle edge case where file might be 'undefined' string or empty
    return { message: 'No file selected or file is empty.', error: true, file: null };
  }
  
  const validatedFields = fileSchema.safeParse({ file: uploadedFile });

  if (!validatedFields.success) {
    const errors = validatedFields.error.flatten().fieldErrors;
    return {
      message: errors.file?.[0] || "Invalid file.",
      error: true,
      file: null,
    };
  }
  
  try {
    const newFileEntry = await addFile({
      name: uploadedFile.name,
      type: uploadedFile.type || 'application/octet-stream',
      size: uploadedFile.size,
    });

    // Save the file to the CDN folder
    const cdnFolderPath = path.join(process.cwd(), 'CDN');
    if (!fs.existsSync(cdnFolderPath)) {
      fs.mkdirSync(cdnFolderPath, { recursive: true });
    }

    const filePathInCdn = path.join(cdnFolderPath, `${newFileEntry.id}.${newFileEntry.extension}`);
    const fileBuffer = Buffer.from(await uploadedFile.arrayBuffer());
    fs.writeFileSync(filePathInCdn, fileBuffer);

    revalidatePath('/browse'); 
    revalidatePath(`/files/${newFileEntry.id}`);
    revalidatePath(`/cdn/${newFileEntry.id}`);

    return { message: `File "${newFileEntry.name}" uploaded and saved successfully!`, error: false, file: newFileEntry };
  } catch (e) {
    console.error("Upload error:", e);
    let errorMessage = 'Failed to upload file. Please try again.';
    if (e instanceof Error) {
        errorMessage = `Failed to upload file: ${e.message}`;
    }
    return { message: errorMessage, error: true, file: null };
  }
}
